package refactor;



/**
 * Skelettimplementierung fr Interface {@link Named}.
 *
 * @layer<typesys>
 * @author Stephan Klapproth
 *
 */
public abstract class AbstractNamed implements Named {

    protected String name;

    /**
     * Gibt den Namen des Objekts zurck.
     * 
     * @return den Namen des Objekts.
     * @see Named#getName()
     */
    public String getName() {
        return name;
    }

    /**
     * Gibt den vollqualifizierten Namen des Objekts zurck.
     * <p>
     * Muss in den Unterklassen von <code>AbstractNamed</code> implementiert 
     * werden.
     * 
     * @return den vollstndig qualifizierten Namen des Objekts.
     * @see Named#getFullName()
     */
    public abstract String getFullName();

    /**
     * Vergleicht das Objekt mit <code>anObj</code>.
     * <p>
     * @return <code>True</code>, wenn die vollqualifizierten Namen 
     * bereinstimmen, sonst <code>False</code>.
     * 
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object anObj) {
        if (((AbstractNamed) this) == anObj) {
            return true;
        }

        if (!(anObj instanceof Named)) {
            return false;
        }

        return ((Named) anObj).getFullName().equals(getFullName());
    }

    /**
     * Gibt den Hashcode der Objektinstanz zurck.
     * <p>
     * Der Hashcode wird aus dem vollqualifizierten Namen berechnet.
     * 
     * @return Hashcode des Objekts.
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return getFullName().hashCode();
    }

    /**
     * Gibt den vollqualifizierten Namen des Objekts zurck.
     * <p>
     * @return <code>String</code> Reprsentation des Objekts.
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return getFullName();
    }
}
