package refactor;



/**
 * BlockScope (z. B. for, while, do-while, try-catch, ... Blcke).
 * 
 * @layer<typesys>
 * @author Stephan Klapproth
 * 
 */
abstract interface BlockScope$$typesys extends Scope, Declaration {

    /**
     * Gibt alle eingebetteten BlockScopes zurck.
     * 
     * @return BlockScopes.
     */
    public BlockScope[] getBlockScopes();

    /**
     * Gibt alle Methodenaufrufe zurck, die sich in diesem Scope befinden.
     * <p>
     * Das Ergebnis enthlt auch die Methodenaufrufe, die sich in den
     * geschachtelten Blcken befinden.
     * <p>
     * Falls sich in diesem Scope keine Methodenaufrufe befinden, wird ein Array
     * der Lnge 0 zurckgegeben.
     * 
     * @return alle Methodenaufrufe dieses BlockScopes.
     */
    public MethodInvocation[] getMethodInvocations();

    /**
     * Gibt alle Klassenreferenzen zurck, die sich in diesem Scope befinden.
     * <p>
     * Das Ergebnis enthlt auch die Referenzen, die sich in den
     * geschachtelten Blcken befinden.
     * <p>
     * Falls sich in diesem Scope keine Methodenaufrufe befinden, wird ein Array
     * der Lnge 0 zurckgegeben.
     * 
     * @return alle Klassenreferenzen dieses BlockScopes.
     */
    public ClassReference[] getClassReferences();

    /**
     * Gibt alle Zugriffe auf Variablen zurck.
     * <p>
     * Das Ergebnis enthlt auch alle Zugriffe, die sich in geschachtelten
     * Blcken befinden.
     * <p>
     * Falls sich in diesem Scope keine Variablenzugriffe befinden, wird ein
     * Array der Lnge 0 zurckgegeben.
     * 
     * @return alle Variablenzugriffe dieses BlockScopes.
     */
    public FieldAccess[] getFieldAccesses();
    
    /**
     * Gibt alle lokalen Variablen zurck, die in diesem Blockscope deklariert
     * wurden.
     * <p>
     * Dazu gehren auch die lokalen Variablen von eingebetteten BlockScopes.
     * <p>
     * Falls keine Variablendeklaration existiert, wird ein Array der Lnge 0
     * zurckgegeben.
     * 
     * @return alle lokalen Variablen dieses Blocks.
     */
    public FieldInfo[] getFields();
}



/**
 * @layer<refactoring_extrstmts>
 * @author Stephan Klapproth
 * 
 */
 interface BlockScope extends BlockScope$$typesys {

    public void move(BlockCScope aDestination);
    
}
