package refactor;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import mixin.AST_Class;
import mixin.AST_Imports;
import mixin.AST_TypeNameList;
import mixin.AstCursor;
import mixin.AstNode;
import mixin.AstToken;
import mixin.ClsBody;
import mixin.ExtClause;
import mixin.ExtendsClause;
import mixin.ImplClause;
import mixin.ImportDeclaration;
import mixin.ImpQual;
import mixin.IntExtClauseC;
import mixin.ModTypeDecl;
import mixin.program;
import mixin.TName;
import mixin.TypeDeclaration;
import mixin.UmInterDecl;
import mixin.UmodClassDecl;
import mixin.UnmodifiedInterfaceDeclaration;
import mixin.UnmodifiedTypeDeclaration;
import mixin.Ute;
import mixin.VarAssignC;
import mixin.VarDecl;
import java.io.FileNotFoundException;
import java.io.PrintWriter;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import mixin.AstProperties;

import mixin.AST_Modifiers;
import mixin.AST_ModifiersElem;
import mixin.ModPublic;

import mixin.AST_FieldDecl;
import mixin.AST_FieldDeclElem;
import mixin.AstOptNode;
import mixin.MethodDcl;

import java.io.StringReader;

import mixin.ParseException;
import mixin.Parser;

import mixin.FldVarDec;

import mixin.AST_QualifiedName;
import mixin.PackageDeclaration;




/**
 * Implementierung von ClassInfo, die den AST zur Ermittlung der Informationen
 * nutzt.
 * 
 * @layer<typesys>
 * @author Stephan Klapproth
 * 
 */
abstract class ClassInfoAst$$typesys extends AbstractClassInfo implements AstElement, Modified {

    LayerInfo _layer;

    private File srcFile;

    private program astProgram;

    protected ModifierInfo[] modifier;

    private Map fields;

    private Map methods;

    private Map interfaces;

    private List methodInvocations;

    private List fieldAccesses;
    
    private List classReferences;

    private SymTab SYMTAB;

    /**
     * Konstruktor.
     * 
     * @param aSource
     *            die Quelldatei.
     * @param aProgram
     *            Ast <code>program</code> node.
     */
    public ClassInfoAst$$typesys(File aSource, program aProgram) {
        srcFile = aSource;
        astProgram = aProgram;
    }
    
    private void initNestedDeclarations() {
    	if ( SYMTAB == null )
	    	SYMTAB = SymTab.getInstance(TypesysUtil.getTypeSystem( ((ClassInfoAst) ((ClassInfoAst) this)) ));
        fields = new HashMap();
        methods = new HashMap();

        SYMTAB.enterScope(((ClassInfoAst) this));
        new ClsBodyDeclVisitor(TypesysUtil.getTypeSystem( ((ClassInfoAst) this) )).visit(astProgram);
        SYMTAB.leaveScope();
    }
    
    private void initReferences() {
        if ( SYMTAB == null )
	    	SYMTAB = SymTab.getInstance(TypesysUtil.getTypeSystem( ((ClassInfoAst) ((ClassInfoAst) this)) ));
        methodInvocations = new ArrayList();
        fieldAccesses = new ArrayList();
        classReferences = new ArrayList();
    
        ReferenceResolver tmpRes = new ReferenceResolver();
        for (int i = 0; i < getFields().length; i++) {
            VarDecl tmpDecl = ((FieldInfoAst) getFields()[i]).getVarDecl();
            if (tmpDecl.getVarAssign() instanceof VarAssignC) {
                tmpRes.resolveVarAssign((VarAssignC) tmpDecl.getVarAssign(), ((ClassInfoAst) this));
            }
        }
        SYMTAB.enterScope(((ClassInfoAst) this));
        SYMTAB.declare(tmpRes.getFieldAccesses());
        SYMTAB.declare(tmpRes.getMethodInvocations());
        SYMTAB.declare(tmpRes.getClassReferences());
        SYMTAB.leaveScope();
    }

    /**
     * Gibt den AST <code>program</code> node zurck.
     * 
     * @return AstNode program.
     */
    public AstNode getAstNode() {
        return astProgram;
    }

    public int startLineNum() {
        AstToken tmpTok = ClassInfoAst.getFirstClass(astProgram).findToken();
        if (tmpTok == null) {
            throw new TypesysException("Can't get the first token of "
                    + ClassInfoAst.getFirstClass(astProgram).className());
        }
        return tmpTok.lineNum();
    }

    public int endLineNum() {
        AstToken tmpTok = TypesysUtil.findLastToken(ClassInfoAst
                .getFirstClass(astProgram));

        if (tmpTok == null) {
            throw new TypesysException("Can't get the last token of "
                    + ClassInfoAst.getFirstClass(astProgram).className());
        }
        return tmpTok.lineNum();
    }

    public ModifierInfo[] getModifier() {
        if ( modifier == null ) {
            initModifier();
        }
        return modifier;
    }
    
    Map getMethodMap() {
        if (methods == null) {
            initNestedDeclarations();
        }
        return methods;
    }

    public MethodInfo[] getMethods() {
        return (MethodInfo[]) getMethodMap().values().toArray(
                new MethodInfo[getMethodMap().size()]);
    }

    ClsBody getClsBody() {
        AstCursor c = new AstCursor();
        for (c.First(getFirstClass(astProgram)); c.More(); c.PlusPlus()) {
            if (c.node instanceof ClsBody) {
                return (ClsBody) c.node;
            }
        }
        return null;
    }

    private Map getFieldsMap() {
        if (fields == null) {
            initNestedDeclarations();
        }
        return fields;
    }

    public FieldInfo[] getFields() {
        return (FieldInfo[]) getFieldsMap().values().toArray(
                new FieldInfo[getFieldsMap().size()]);
    }

    static TypeDeclaration getFirstClass(program aProgram) {
        AST_Class tmpClass = aProgram.getAST_Class();
        if (tmpClass != null) {
            AstCursor c = new AstCursor();
            c.FirstElement(tmpClass);
            if (c.node instanceof TypeDeclaration) {
                return (TypeDeclaration) (c.node);
            }
        }
        throw new TypesysException("No class found.");
    }

    UnmodifiedTypeDeclaration getUtd() {
        if (!(getFirstClass(astProgram) instanceof ModTypeDecl)) {
            throw new TypesysException(srcFile.getName() + " is an invalid class. No ModTypeDecl found.");
        }

        return ((ModTypeDecl) getFirstClass(astProgram))
                .getUnmodifiedTypeDeclaration();
    }

    /*
     * (non-Javadoc)
     * 
     * @see AbstractNamed#getFullName()
     */
    public String getFullName() {
        return getName();
    }

    /*
     * (non-Javadoc)
     * 
     * @see AbstractNamed#getName()
     */
    public String getName() {
        if (getUtd() instanceof UmodClassDecl) {
            return ((UmodClassDecl) getUtd()).getQName().GetName();
        } else if (getUtd() instanceof UmInterDecl) {
            return ((UmInterDecl) getUtd()).getQName().GetName();
        }
        return "";
    }

    /**
     * Gibt die Quelldatei der Klasse zurck.
     * 
     * @return Quelldatei.
     */
    public File getSrcFile() {
        return srcFile;
    }

    /*
     * (non-Javadoc)
     * 
     * @see Scope#addDeclaration(Declaration)
     */
    public void addDeclaration(Declaration aDeclaration) {
        aDeclaration.setContext(((ClassInfoAst) this));

        if (aDeclaration instanceof ModifiedFieldInfoAst) {
            getFieldsMap().put(aDeclaration.getName(), aDeclaration);
        } else if (aDeclaration instanceof MethodInfoAst) {
            getMethodMap().put(((MethodInfo) aDeclaration).signature(), aDeclaration);
        } else if (aDeclaration instanceof MethodInvocation) {
            getMethodInvocationList().add(aDeclaration);
        } else if (aDeclaration instanceof FieldAccess) {
            getFieldAccessList().add(aDeclaration);
        } else if (aDeclaration instanceof ClassReference) {
            getClassReferenceList().add(aDeclaration);
        } else {
            throw new IllegalArgumentException(aDeclaration.getName()
                    + "can't be added to " + getFullName());
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see Scope#getContext()
     */
    public Scope getContext() {
        return _layer;
    }

    /*
     * (non-Javadoc)
     * 
     * @see Scope#setContext(Scope)
     */
    public void setContext(Scope aScope) {
        if (!(aScope instanceof LayerInfo)) {
            throw new IllegalArgumentException("Class can't be added to scope "
                    + aScope.getFullName() + "!");
        }
        _layer = (LayerInfo) aScope;
    }

    /**
     * Gibt alle importierten Klassen zurck.
     * 
     * @return String Array mit allen importierten Klassen.
     */
    public String[] getImports() {

        // TODO auch die Imports der Basisklassen zurckgeben.

        AST_Imports tmpImports = astProgram.getAST_Imports();

        if (tmpImports == null || tmpImports.arg.length == 0) {
            return new String[0];
        }

        List tmpResult = new ArrayList();

        AstCursor c = new AstCursor();
        for (c.FirstElement(tmpImports); c.MoreElement(); c.NextElement()) {
            ImportDeclaration tmpImpDecl = (ImportDeclaration) c.node;

            if (!(tmpImpDecl instanceof ImpQual)) {
                continue;
            }
            String tmpImport = ((ImpQual) tmpImpDecl).getAST_QualifiedName()
                    .GetName();
            tmpImport += (((ImpQual) tmpImpDecl).getDotTimes() == null) ? ""
                    : ((ImpQual) tmpImpDecl).getDotTimes().toString();
            tmpResult.add(tmpImport);

        }

        return (String[]) tmpResult.toArray(new String[tmpResult.size()]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see ClassInfo#getInterfaces()
     */
    public ClassInfo[] getInterfaces() {
        if (interfaces == null) {
            initInterfaces();
        }
        return (ClassInfo[]) interfaces.values().toArray(
                new ClassInfo[interfaces.size()]);
    }

    private void initInterfaces() {
        interfaces = new HashMap();

        if (getAST_TypeNameList() == null) {
            return;
        }

        AstCursor c = new AstCursor();
        for (c.FirstElement(getAST_TypeNameList()); c.MoreElement(); c
                .NextElement()) {
            TName tmpElem = (TName) c.node;
            ClassInfo tmpType = TypeFactory.valueOf(tmpElem.GetName(), ((ClassInfoAst) this));
            interfaces.put(tmpType.getFullName(), tmpType);
        }
    }

    protected AST_TypeNameList getAST_TypeNameList() {
        if (getUtd() instanceof UmInterDecl) {
            return getAST_TypeNameList((UmInterDecl) getUtd());
        } else if (getUtd() instanceof UmodClassDecl) {
            return getAST_TypeNameList((UmodClassDecl) getUtd());
        }

        return null;
    }

    private AST_TypeNameList getAST_TypeNameList(UmodClassDecl aDecl) {
        if (!(aDecl.getImplementsClause() instanceof ImplClause)) {
            return null;
        }
        return ((ImplClause) aDecl.getImplementsClause()).getAST_TypeNameList();
    }

    private AST_TypeNameList getAST_TypeNameList(UmInterDecl aDecl) {
        if (!(aDecl.getIntExtClause() instanceof IntExtClauseC)) {
            return null;
        }
        return ((IntExtClauseC) aDecl.getIntExtClause()).getAST_TypeNameList();
    }

    /*
     * (non-Javadoc)
     * 
     * @see ClassInfo#isInterface()
     */
    public boolean isInterface() {
        return (getUtd() instanceof UnmodifiedInterfaceDeclaration);
    }

    protected void initModifier() {
        modifier = TypesysUtil
                .getModifierFromAst(((ModTypeDecl) getFirstClass(astProgram))
                        .getAST_Modifiers());
    }

    /*
     * (non-Javadoc)
     * 
     * @see ClassInfo#getSuperClass()
     */
    public ClassInfo getSuperClass() {
        if (getUtd() instanceof UmodClassDecl) {
            ExtendsClause tmpExt = ((UmodClassDecl) getUtd())
                    .getExtendsClause();
            if (tmpExt instanceof ExtClause) {
                return TypeFactory.valueOf(((ExtClause) tmpExt)
                        .getAST_QualifiedName().GetName(), ((ClassInfoAst) this));
            }
        }
        return new ClassInfoReflAdapter(Object.class);
    }

    /**
     * Statische Fabrikmethode zur Erzeugung von {@link ClassInfo} Instanzen.
     * <p>
     * Die Fabrikmethode entscheidet an Hand des AST, ob es sich um eine
     * Basisklasse oder eine Refinementklasse handelt.
     * 
     * @param aSource
     *            die Quelldatei.
     * @param aProgram
     *            AstNode.
     * @return Basisklasse oder Refinementklasse, je nach AST.
     */
    public static ClassInfo newInstance(File aSource, program aProgram) {
        if (isBaseClass(getFirstClass(aProgram))) {
            return new ClassInfoAst(aSource, aProgram);
        } else {
            return new RefinementInfoAst(aSource, aProgram);
        }
    }

    private static boolean isBaseClass(TypeDeclaration aDecl) {
        if (!(aDecl instanceof ModTypeDecl)) {
            throw new TypesysException("Not a valid class.");
        }
        UnmodifiedTypeDeclaration tmpDecl = ((ModTypeDecl) aDecl)
                .getUnmodifiedTypeDeclaration();
        return !(tmpDecl instanceof Ute);
    }

    public MethodInfo findMethod(String aName, ClassInfo[] someParams) {

        MethodInfo tmpMethod = getMethod(aName, someParams);
        if (tmpMethod != null) {
            return tmpMethod;
        }

        tmpMethod = getMethodInSuperClass(aName, someParams);
        if (tmpMethod != null) {
            return tmpMethod;
        }

        return getMethodInInterface(aName, someParams);
    }

    MethodInfo getMethod(String aName, ClassInfo[] someParams) {
        String tmpSignature = AbstractMethodInfo
                .getSignature(aName, someParams);
        if (getMethodMap().containsKey(tmpSignature)) {
            return (MethodInfo) getMethodMap().get(tmpSignature);
        }
        return null;
    }

    private MethodInfo getMethodInSuperClass(String aName,
            ClassInfo[] someParams) {
        if (getSuperClass() != null) {
            return getSuperClass().findMethod(aName, someParams);
        }
        return null;
    }

    private MethodInfo getMethodInInterface(String aName, ClassInfo[] someParams) {
        for (int i = 0; i < getInterfaces().length; i++) {
            MethodInfo tmpMethod = getInterfaces()[i].findMethod(aName,
                    someParams);
            if (tmpMethod != null) {
                return tmpMethod;
            }
        }
        return null;
    }

    public FieldInfo getField(String aName) {
        return (FieldInfo) getFieldsMap().get(aName);
    }
    
    private List getMethodInvocationList() {
        if (methodInvocations == null) {
            initReferences();
        }
        return methodInvocations;
    }

    /**
     * Gibt alle Methodenaufrufe zurck, die in der Klasse auftreten.
     * 
     * @return alle Methodenaufrufe in der Klasse.
     */
    public MethodInvocation[] getMethodInvocations() {
    
        List tmpInvocations = new ArrayList();
        tmpInvocations.addAll(getMethodInvocationList());

        for (int i = 0; i < getMethods().length; i++) {
            tmpInvocations.addAll(Arrays
                    .asList(((MethodInfoAst) getMethods()[i])
                            .getMethodInvocations()));
        }

        return (MethodInvocation[]) tmpInvocations
                .toArray(new MethodInvocation[tmpInvocations.size()]);
    }
    
    /**
     * Gibt alle Referenzen auf Klassen zurck, die in der Klasse auftreten.
     * 
     * @return alle Klassenreferenzen in der Klasse.
     */
    public ClassReference[] getClassReferences() {
    
        List tmpReferences = new ArrayList();
        tmpReferences.addAll(getClassReferenceList());

        for (int i = 0; i < getMethods().length; i++) {
            tmpReferences.addAll(Arrays
                    .asList(((MethodInfoAst) getMethods()[i])
                            .getClassReferences()));
        }

        return (ClassReference[]) tmpReferences
                .toArray(new ClassReference[tmpReferences.size()]);
    }
    
    private List getClassReferenceList() {
        if (classReferences == null) {
            initReferences();
        }
        return classReferences;
    }
    
    
    private List getFieldAccessList() {
        if (fieldAccesses == null) {
            initReferences();
        }
        return fieldAccesses;
    }

    /**
     * Gibt alle Variablenzugriffe zurck, die in der Klasse auftreten.
     * 
     * @return alle Variablenzugriffe in der Klasse.
     */
    public FieldAccess[] getFieldAccesses() {
        List tmpAccesses = new ArrayList();
        tmpAccesses.addAll(getFieldAccessList());

        for (int i = 0; i < getMethods().length; i++) {
            tmpAccesses.addAll(Arrays
                    .asList(((MethodInfoAst) getMethods()[i])
                            .getFieldAccesses()));
        }

        return (FieldAccess[]) tmpAccesses
                .toArray(new FieldAccess[tmpAccesses.size()]);
    }

    public boolean isPublic() {
        return Arrays.asList(getModifier()).contains(
                ModifierInfo.valueOf(ModifierInfo.PUBLIC));
    }

    public boolean isPrivate() {
        return Arrays.asList(getModifier()).contains(
                ModifierInfo.valueOf(ModifierInfo.PRIVATE));
    }

    public boolean isProtected() {
        return Arrays.asList(getModifier()).contains(
                ModifierInfo.valueOf(ModifierInfo.PROTECTED));
    }

    public boolean isPackagePrivate() {
    	return !(isPublic() || isPrivate() || isProtected());
    }
}




/**
 * @layer<refactoring_base>
 * @author Stephan Klapproth
 * 
 */
 abstract class ClassInfoAst$$refactoring_base extends  ClassInfoAst$$typesys  implements Saveable {

    private static final Log LOG = LogFactory
    		.getLog("refactoring.base.ClassInfoAst");

   
    /*
     * (non-Javadoc)
     * 
     * @see Saveable#save()
     */
    public void save() {
		if (getSrcFile().getParentFile() != null) {
			getSrcFile().getParentFile().mkdirs();
		}
        PrintWriter tmpWriter;
        try {
            tmpWriter = new PrintWriter(getSrcFile());
            AstProperties props = new AstProperties();
            props.setProperty("output", tmpWriter);
            getAstNode().print(props);
            tmpWriter.flush();
            tmpWriter.close();
        } catch (FileNotFoundException e) {
            LOG.error("Saving file failed.", e);
        }
    }
      // inherited constructors



    /**
     * Konstruktor.
     * 
     * @param aSource
     *            die Quelldatei.
     * @param aProgram
     *            Ast <code>program</code> node.
     */
    public ClassInfoAst$$refactoring_base ( File aSource, program aProgram ) { super(aSource, aProgram); }
}




/**
 * 
 * @layer<refactoring_changemodifier>
 * @author Stephan Klapproth
 * @author Stephan Kauschka
 * 
 */
 abstract class ClassInfoAst$$refactoring_changemodifier extends  ClassInfoAst$$refactoring_base  {

    //Sucht den Node AST_Modifiers und bergibt ihn.
    //Wenn dieser nicht existiert, wird er erstellt und in den AST eingebunden.
    public AST_Modifiers getAstModifiers(){

	AstNode tmpNode = getAstNode();

	//Suche den Knoten ModTypeDecl
	AstCursor c = new AstCursor();
	for (c.First(tmpNode); c.More(); c.PlusPlus()) {
	    if(c.node instanceof ModTypeDecl){
		tmpNode = c.node;
		break;
	    }
	}

	//Suche den Knoten AST_Modifiers
	if(tmpNode.arg[0].arg != null && tmpNode.arg[0].arg[0]!=null)
	    return (AST_Modifiers)tmpNode.arg[0].arg[0];

	//Wenn der Knoten nicht vorhanden ist erstelle ihn und binde ihn ein
	tmpNode = tmpNode.arg[0];
	tmpNode.arg = new AstNode[]{new AST_Modifiers()};
	tmpNode.arg[0].up = tmpNode;
	tmpNode.arg[0].arg = new AstNode[]{null};
	return (AST_Modifiers)tmpNode.arg[0];
    }

    //ndert nur die Sichtbarkeits-Modifier
    public void changeVisibility(ModifierInfo[] someModifier)
    throws TransformationException{

	changeVisibilityLocal(someModifier);

	AST_Modifiers astModifiersNode = getAstModifiers();
	AstCursor cursor = new AstCursor();
	if(astModifiersNode.arg!=null && astModifiersNode.arg[0]!=null)
	    for (cursor.First(astModifiersNode); cursor.More(); cursor.PlusPlus()) {
		//Wenn schon ein Public-Modifier vorhanden war
		if(cursor.node instanceof ModPublic){
		    //wenn auf packageprivate gewechselt werden soll
		    if(someModifier[0]==null){
			String whitespace = ((AstToken)cursor.node.tok[0]).white_space;

			//nchste AstToken suchen um whitespace zu setzen
			AstCursor c = new AstCursor();
			AstNode start = cursor.node.up.right!= null ? cursor.node.up.right : astModifiersNode.up.right;
			for (c.First(start); c.More(); c.PlusPlus()) {
			    if(c.node.tok != null && c.node.tok[0]!=null){
				((AstToken)c.node.tok[0]).white_space = whitespace;
				break;
			    }
			}

			//Delete the Modifier
			ModifierFactory.deleteNode(cursor.node.up);
			return;
		    }
		    //wenn public bleiben soll
		    return;
		}
	    }

	//Wenn die Klasse packageprivate war muss ein neuer AST_ModifiersElem-Knoten
	//erstellt werden

	//aber nur wenn nicht auf packageprivate gewechselt erden soll
	if(someModifier[0]==null)
	    return;

	AST_ModifiersElem astModifiersElemNode = new AST_ModifiersElem();

	String whitespace = "";
	//nchste AstToken suchen um whitespace zu setzen
	AstCursor c = new AstCursor();
	for (c.First(astModifiersNode.up.up); c.More(); c.PlusPlus()) {
	    if(c.node.tok != null && c.node.tok[0]!=null){
		whitespace = ((AstToken)c.node.tok[0]).white_space;
		((AstToken)c.node.tok[0]).white_space = " ";
		break;
	    }
	}

	astModifiersElemNode.arg = new AstNode[] {ModifierFactory.getModifier((Modified)((ClassInfoAst) ((ClassInfoAst) ((ClassInfoAst) this))),"public",whitespace)};
	astModifiersElemNode.arg[0].up = astModifiersElemNode;

	ModifierFactory.insertAsFirst(astModifiersNode,astModifiersElemNode);
    }

    //ndert nur die Sichtbarkeits-Modifier des modifier-Felds des ClassInfoAst
    private void changeVisibilityLocal(ModifierInfo[] someModifier)
    throws TransformationException {

	//Eine Klasse kann nur einen Sichtbarkeits-Modifier besitzen
	if(someModifier.length!=1)
	    throw new TransformationException("only one access modifier is allowed.");

	//nur Public-Modifier und packageprivate sind in dieser Methode zugelassen
	if(someModifier[0]!=null && !someModifier[0].equals(ModifierInfo.valueOf(ModifierInfo.PUBLIC)))
	    throw new TransformationException(someModifier[0] +
	    " is no public modifier.");

	//wenn es vorher keine Modifier gab
	if(modifier == null){
	    if(someModifier[0]!=null)
		modifier = someModifier;
	    return;
	}

	for(int j=0; j<modifier.length; j++)
	    //Wenn schon ein Sichtbarkeits-Modifier vorhanden war
	    if(modifier[j].equals(ModifierInfo.valueOf(ModifierInfo.PUBLIC))){
		//Wenn der neue Modifier auch public ist
		if(someModifier[0]!=null)
		    return;
		else{
		    //wenn auf packageprivate gewechselt werden soll
		    int length = modifier.length-1;
		    ModifierInfo[] tmpModifier = new ModifierInfo[length];
		    int l=0;
		    for(int k=0; k<modifier.length; k++)
			if(!modifier[k].equals(ModifierInfo.valueOf(ModifierInfo.PUBLIC))){
			    tmpModifier[l] = modifier[k];
			    l++;
			}
		    modifier = tmpModifier;
		    return;
		}
	    }

	//wenn vorher packageprivate war und der neue Modifier auch null
	if(someModifier[0]==null)
	    return;

	int length = modifier.length+1;
	ModifierInfo[] tmpModifier = new ModifierInfo[length];

	for(int j=0; j<modifier.length; j++)
	    tmpModifier[j+1] = modifier[j];
	tmpModifier[0] = someModifier[0];

	modifier = tmpModifier;
    }


    public void changeModifier(ModifierInfo[] someModifier)
    	throws TransformationException {
    }
      // inherited constructors



    /**
     * Konstruktor.
     * 
     * @param aSource
     *            die Quelldatei.
     * @param aProgram
     *            Ast <code>program</code> node.
     */
    public ClassInfoAst$$refactoring_changemodifier ( File aSource, program aProgram ) { super(aSource, aProgram); }

}




/**
 * @layer<refactoring_createmethod>
 * @author Stephan Klapproth
 * 
 */
 abstract class ClassInfoAst$$refactoring_createmethod extends  ClassInfoAst$$refactoring_changemodifier  {

    private AST_FieldDecl getFieldDecl() throws TransformationException {
        ClsBody tmpBody = getClsBody();
        if (tmpBody == null) {
            throw new TransformationException("No class body found.");
        }
        AST_FieldDecl tmpDecl = tmpBody.getAST_FieldDecl();
        if (tmpDecl == null) {
            tmpBody.setParms((AstToken) tmpBody.tok[0],
                    new AstOptNode().setParms(new AST_FieldDecl()),
                    (AstToken) tmpBody.tok[1]);
        }
        return tmpBody.getAST_FieldDecl();
    }

    MethodInfo addMethod(MethodDcl aDecl) throws TransformationException {
        getFieldDecl().add(new AST_FieldDeclElem().setParms(aDecl));
        MethodInfo tmpMethod = new MethodInfoAst(aDecl);
        addDeclaration(tmpMethod);
        return tmpMethod;
    }
      // inherited constructors



    /**
     * Konstruktor.
     * 
     * @param aSource
     *            die Quelldatei.
     * @param aProgram
     *            Ast <code>program</code> node.
     */
    public ClassInfoAst$$refactoring_createmethod ( File aSource, program aProgram ) { super(aSource, aProgram); }
}




/**
 * @layer<refactoring_createclass>
 * @author Stephan Klapproth
 * 
 */
 abstract class ClassInfoAst$$refactoring_createclass extends  ClassInfoAst$$refactoring_createmethod  {

    public static ClassInfo createEmptyClass(LayerInfo aLayer,
            ModifierInfo[] someModifier, String aClassName,
            ClassInfo aSuperClass, ClassInfo[] someInterfaces, File aFile)
            throws TransformationException {

        if (aLayer == null) {
            throw new NullPointerException("aLayer cannot be null.");
        }

        ClassInfo tmpExistingClass = aLayer.findClass(aClassName);
        if (tmpExistingClass != null 
                && tmpExistingClass.getContext().equals(aLayer)) {
            throw new TransformationException(aClassName
                    + " already exists in the layer "
                    + tmpExistingClass.getContext() + ".");
        }

        String tmpCode = new AstStringUtil().getBaseClass(aLayer.getName(),
                aClassName, aSuperClass, someInterfaces, someModifier);
        ClassInfoAst tmpClass = new ClassInfoAst(aFile, (program) parse(tmpCode));
        aLayer.addDeclaration(tmpClass);
        return tmpClass;

    }

    static AstNode parse(String aCode) {
        try {
            AstNode tmpNode = Parser.getInstance(new StringReader(aCode))
                    .parseAll();
            return tmpNode;

        } catch (ParseException e) {
            return null;
        }
    }
      // inherited constructors



    /**
     * Konstruktor.
     * 
     * @param aSource
     *            die Quelldatei.
     * @param aProgram
     *            Ast <code>program</code> node.
     */
    public ClassInfoAst$$refactoring_createclass ( File aSource, program aProgram ) { super(aSource, aProgram); }
}



/**
 * @layer<refactoring_extrmethod>
 * @author Stephan Klapproth
 * 
 */
 abstract class ClassInfoAst$$refactoring_extrmethod extends  ClassInfoAst$$refactoring_createclass  {

    public void removeMethod(String aSignature) {
        getMethodMap().remove(aSignature);
    }
      // inherited constructors



    /**
     * Konstruktor.
     * 
     * @param aSource
     *            die Quelldatei.
     * @param aProgram
     *            Ast <code>program</code> node.
     */
    public ClassInfoAst$$refactoring_extrmethod ( File aSource, program aProgram ) { super(aSource, aProgram); }
    
}




/**
 * @layer<refactoring_extrstmts>
 * @author Stephan Klapproth
 * 
 */
 abstract class ClassInfoAst$$refactoring_extrstmts extends  ClassInfoAst$$refactoring_extrmethod  {
	
    private String buildFieldDecl(ModifierInfo[] someModifier, ClassInfo aType,
            String aName) {

        StringBuilder tmpStr = new StringBuilder("\n\n");
        tmpStr.append(RefUtil.indent(1));
        if (someModifier != null && someModifier.length > 0) {
            for (int i = 0; i < someModifier.length; i++) {
                if (i > 0) {
                    tmpStr.append(" ");
                }
                tmpStr.append(someModifier[i]);
            }
            tmpStr.append(" ");
        }
        tmpStr.append(aType.getFullName());
        tmpStr.append(" ");
        tmpStr.append(aName);
        tmpStr.append(";");
        return tmpStr.toString();
    }

    FieldInfo addField(ModifierInfo[] someModifier, ClassInfo aType,
            String aName) {
        if (aType == null || aName == null || someModifier == null) {
            throw new IllegalArgumentException("All parameters are mandatory!");
        }
        AST_FieldDecl tmpDecls = AST_FieldDecl.MakeAST(buildFieldDecl(
                someModifier, aType, aName));
        FldVarDec tmpDec = (FldVarDec) tmpDecls.last.arg[0];

        addFieldDecl(tmpDecls);
        
        ModifiedFieldInfoAst tmpField = new ModifiedFieldInfoAst(
                tmpDec.getAST_TypeName(), (VarDecl) tmpDec.getAST_VarDecl()
                .last.arg[0], tmpDec.getAST_Modifiers());
        addDeclaration(tmpField);
        return tmpField;
    }
    
    private void addFieldDecl(AST_FieldDecl aDecl) {
        if (getClsBody().getAST_FieldDecl() == null) {
            getClsBody().arg[0] = new AstOptNode().setParms(aDecl);
        } else {
            getClsBody().getAST_FieldDecl().add(aDecl);
        }
    }
    
    void addConstructor(ClassInfo[] someParamTypes, String[] someParamNames) {
        StringBuilder tmpConstr = new StringBuilder("\n");
        tmpConstr.append(RefUtil.indent(1));
        tmpConstr.append(getName());
        tmpConstr.append("(");
        for (int i = 0; i < someParamTypes.length; i++) {
            if (i > 0) {
                tmpConstr.append(", ");
            }
            tmpConstr.append(someParamTypes[i].getFullName());
            tmpConstr.append(" ");
            tmpConstr.append(someParamNames[i]);
        }
        tmpConstr.append(") {\n");
        for (int i = 0; i < someParamTypes.length; i++) {
            tmpConstr.append(RefUtil.indent(2));
            tmpConstr.append("this.");
            tmpConstr.append(someParamNames[i]);
            tmpConstr.append(" = ");
            tmpConstr.append(someParamNames[i]);
            tmpConstr.append(";\n");
        }
        tmpConstr.append("\n");
        tmpConstr.append(RefUtil.indent(1));
        tmpConstr.append("}");
        
        AST_FieldDecl tmpDecl = AST_FieldDecl.MakeAST(tmpConstr.toString());
        addFieldDecl(tmpDecl);
    }
      // inherited constructors



    /**
     * Konstruktor.
     * 
     * @param aSource
     *            die Quelldatei.
     * @param aProgram
     *            Ast <code>program</code> node.
     */
    public ClassInfoAst$$refactoring_extrstmts ( File aSource, program aProgram ) { super(aSource, aProgram); }
}




 class ClassInfoAst extends  ClassInfoAst$$refactoring_extrstmts  {

    private boolean delete = false;
    
    public void delete() {
        delete = true;
    }
    
    public void save() {
        if (delete) {
            getSrcFile().delete();
        } else {
            super.save();
        }
    }     

    public void changeLayerName(String aNewLayer) throws TransformationException {
        program tmpNode = (program) getAstNode();
        PackageDeclaration tmpDecl = tmpNode.getPackageDeclaration();
        if ( tmpDecl == null ){
            return;
        }
        tmpDecl.arg[0] = AST_QualifiedName.MakeAST(" " + aNewLayer);
    }
      // inherited constructors



    /**
     * Konstruktor.
     * 
     * @param aSource
     *            die Quelldatei.
     * @param aProgram
     *            Ast <code>program</code> node.
     */
    public ClassInfoAst ( File aSource, program aProgram ) { super(aSource, aProgram); }
}
