package refactor;




/**
 * Implementierung von FieldInfo fr das Feld length von Arrays.
 * Die Extra-Implementierung ist ntig weil dieses Feld nicht durch
 * Reflektion (d.h. via java.lang.Class.getFields() bzw.
 * java.lang.Class.getDeclaredFields()) erreicht und somit 
 * auch keine ModifiedFieldInfoReflAdapter-Instanz angelegt werden kann.
 * 
 * @layer<typesys>
 * @author Stephan Kauschka
 *
 */
class LengthFieldInfoReflAdapter implements FieldInfo {

    private Class clazz;

    public LengthFieldInfoReflAdapter(Class anArrayClass){
		clazz = anArrayClass;
    }

    public ClassInfo getType() {
		return TypeFactory.primitiveType(TypeFactory.INT);
    }

    public Scope getContext() {
		int tmpArrayDim = 0;

        String tmpName = clazz.getCanonicalName();
        while (tmpName.endsWith("[]")) {
            tmpArrayDim++;
            tmpName = tmpName.substring(0, tmpName.length() - 2);
        }

        Class tmpType = clazz.getComponentType();
        while (tmpType.isArray()){
            tmpType = tmpType.getComponentType();
        }

        return new ArrayType(new ClassInfoReflAdapter(tmpType), tmpArrayDim);
    }

    public void setContext(Scope scope) {
		throw new UnsupportedOperationException();
    }

    public String getFullName() {
		return getContext().getFullName() + "." + getName();
    }

    public String getName() {
		return "length";
    }

}
